window.Gleam.initSearch([{"doc":"toy","title":"toy","content":"# Toy\n\nPractical, composable dynamic decoders and validators for Gleam!\n\n[![Package Version](https://img.shields.io/hexpm/v/toy)](https://hex.pm/packages/toy)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/toy/)\n\nCheckout the [quickstart guide](https://hexdocs.pm/toy/quickstart.html) to get started.\n\n```sh\ngleam add toy\n```\n```gleam\nimport gleam/dynamic\nimport gleam/json\nimport toy\n\npub type User {\n  User(name: String, height: Float, age: Int)\n}\n\npub fn user_decoder() {\n  use name &lt;- toy.field(&quot;name&quot;, toy.string)\n  use height &lt;- toy.field(&quot;height&quot;, toy.float |&gt; toy.float_min(0.0))\n  use age &lt;- toy.field(&quot;age&quot;, toy.int |&gt; toy.int_min(0))\n\n  toy.decoded(User(name:, height:, age:))\n}\n\npub fn main() {\n  let assert Ok(data) =\n    json.decode(\n      &quot;\n  {\n    \\&quot;name\\&quot;: \\&quot;Alice\\&quot;,\n    \\&quot;height\\&quot;: 1.7,\n    \\&quot;age\\&quot;: 25\n  }\n  &quot;,\n      dynamic.dynamic,\n    )\n\n  let user = data |&gt; toy.decode(user_decoder())\n}\n```\n\n## Goals\n\nToy aims to provide a simple to use, composable api for decoding and validating\ndynamic data in Gleam. It is inspired by the gleam standard library,\n[decode](https://hexdocs.pm/decode/) and [Zod](https://zod.dev/).\n\nToy strives to satisfy all of the following goals:\n- Composable\n- Good DX\n- Type safe\n- No opaque types\n- Minimize surface for dumb mistakes\n- Return all error messages possible\n- Allow for easy internationalization of error messages\n- Provide validation functions for common use cases\n\n## Why not existing solutions?\n\nIn application development you most of the time decode dynamic values\ninto records (objects in other lanugages).\nIt might be a JSON payload from an API, a configuration file,\nor a database row. So this will be the main focus of this comparison.\n\n### Gleam standard library [gleam_stdlib/dynamic](https://hexdocs.pm/gleam_stdlib/gleam/dynamic.html)\n\n```gleam\npub fn user_decoder_stdlib() {\n  dynamic.decode3(\n    User,\n    dynamic.field(&quot;name&quot;, dynamic.string),\n    dynamic.field(&quot;height&quot;, dynamic.float),\n    dynamic.field(&quot;age&quot;, dynamic.int),\n  )\n}\n```\n\nThe standard library requires two thing from the developer. You have to\nuse the right function, `decode3` in this case, because the user has 3 fields\nand *you have to list the fields in the same order as the record definition*.\n\nThe first point is generally not a problem until you reach 10 fields, because\nthere is no `decode10` function. The second point is a bigger problem.\nWhen developing, it is very common to modify existing records (add/remove fields).\nWhen you do this, it is very easy to mess up the order int the decoder. And if the\ntwo fields you swapped are both strings, the compiler will not catch this mistake.\n**You will end up with bad data in your application**.\n\n### The decode package [decode](https://hexdocs.pm/decode/)\n\n```gleam\npub fn user_decoder_decode() {\n  decode.into({\n    use name &lt;- decode.parameter\n    use height &lt;- decode.parameter\n    use age &lt;- decode.parameter\n    User(name, height, age)\n  })\n  |&gt; decode.field(&quot;name&quot;, decode.string)\n  |&gt; decode.field(&quot;height&quot;, decode.float)\n  |&gt; decode.field(&quot;age&quot;, decode.int)\n}\n```\n\nThe decode package is a bit better than the standard library. Now you can decode\nrecords with any number of fields in the same way easily. However the second\nproblem still persists. The second `decode.field(&quot;height&quot;, decode.float)` call \nhas to match up with the second `use height &lt;- decode.parameter` call, which\nneeds to match up with the right field in the record definition. If you mess\nthis up during refactoring or developing a new feature, **you will end up with\nbad data in your application.**\n\n### Toy\n\n```gleam\npub fn user_decoder_toy() {\n  use name &lt;- toy.field(&quot;name&quot;, toy.string)\n  use height &lt;- toy.field(&quot;height&quot;, toy.float)\n  use age &lt;- toy.field(&quot;age&quot;, toy.int)\n\n  toy.decoded(User(name:, height:, age:))\n}\n```\n\nToy solves both problems. Each field is defined on a single line. And since\ngleam 1.4.0, we can construct the record with the *label shorthand* syntax.\nBy using this syntax the compiler guarantees that the *name* value will be\nassigned to the name field in the record, regardless of the order you specify\nthem in. This makes is much harder to mess up the decoder when refactoring.\n\nBut there is a catch. **Every line of the decoder will be executed on every\ninvocation**. To return errors for all fields at once, we need to continue\ndecoding even after the first error. *But what will be in the `name` variable\nif the `name` field failed to decode?* The answer is a default string value\n(empty string in this case). So you have to make sure to not perform any\nside effects in the decoder, because withing the decoder function you are never\nguaranteed that the data is actually valid.\n\n```gleam\npub fn user_decoder_toy() {\n  use name &lt;- toy.field(&quot;name&quot;, toy.string)\n  // This is bad, you should never perform side effects in the decoder\n  io.println(name)\n  use height &lt;- toy.field(&quot;height&quot;, toy.float)\n  use age &lt;- toy.field(&quot;age&quot;, toy.int)\n\n  toy.decoded(User(name:, height:, age:))\n}\n```\n\nThis is not as scary as it seems. When you are using toy and decode a value with\nit, you will always get valid data. The rule only applies withing the decoder\nitself.\n\nAnd in practice, the rule *don&#39;t perform side effects in the decoder* is much\neasier to follow than the rule *make sure the field order in the decoder always\nmatches the record definition*.\n\n## Usage\n\nHere is a complex example of a decoder, including validation. This should give\nyou a good overview of the capabilities of toy.\n\n```gleam\nimport birl\nimport gleam/dynamic\nimport gleam/io\nimport gleam/json\nimport gleam/option.{type Option}\nimport gleam/order\nimport gleam/result\nimport toy\n\npub type Guest {\n  Guest(name: String, age: Int, special_needs: Option(String))\n}\n\npub type Table {\n  Any\n  Specific(table_number: Int)\n}\n\npub type Reservation {\n  Reservation(\n    person_name: String,\n    email: String,\n    datetime: birl.Time,\n    guests: List(Guest),\n    table: Table,\n    notes: Option(String),\n  )\n}\n\npub fn time_decoder() {\n  toy.string\n  |&gt; toy.try_map(birl.now(), fn(val) {\n    birl.parse(val)\n    |&gt; result.replace_error([toy.ToyError(toy.InvalidType(&quot;DateTime&quot;, val), [])])\n  })\n}\n\npub fn time_future(val: birl.Time) {\n  case birl.compare(val, birl.now()) {\n    order.Gt -&gt; Ok(Nil)\n    _ -&gt;\n      Error([\n        toy.ToyError(\n          toy.ValidationFailed(&quot;date_future&quot;, &quot;DateTime&quot;, birl.to_http(val)),\n          [],\n        ),\n      ])\n  }\n}\n\npub fn reservation_decoder() {\n  use person_name &lt;- toy.field(&quot;person_name&quot;, toy.string |&gt; toy.string_nonempty)\n  use email &lt;- toy.field(&quot;email&quot;, toy.string |&gt; toy.string_email)\n  use datetime &lt;- toy.field(\n    &quot;datetime&quot;,\n    time_decoder() |&gt; toy.refine(time_future),\n  )\n  use guests &lt;- toy.field(\n    &quot;guests&quot;,\n    toy.list({\n      use name &lt;- toy.field(&quot;name&quot;, toy.string)\n      use age &lt;- toy.field(&quot;age&quot;, toy.int |&gt; toy.int_min(18))\n      use special_needs &lt;- toy.optional_field(&quot;special_needs&quot;, toy.string)\n      toy.decoded(Guest(name:, age:, special_needs:))\n    })\n      |&gt; toy.list_nonempty,\n  )\n  use table &lt;- toy.field(&quot;table&quot;, {\n    use type_ &lt;- toy.field(&quot;type&quot;, toy.string)\n\n    case type_ {\n      &quot;specific&quot; -&gt; {\n        use table_number &lt;- toy.field(&quot;table_number&quot;, toy.int)\n        toy.decoded(Specific(table_number:))\n      }\n      &quot;any&quot; -&gt; toy.decoded(Any)\n      _ -&gt; toy.fail(toy.InvalidType(&quot;Table&quot;, type_), Any)\n    }\n  })\n  use notes &lt;- toy.field(&quot;notes&quot;, toy.string |&gt; toy.nullable)\n  toy.decoded(Reservation(\n    person_name:,\n    email:,\n    datetime:,\n    guests:,\n    table:,\n    notes:,\n  ))\n}\n\nconst data = &quot;\n{\n  \\&quot;person_name\\&quot;: \\&quot;Alice\\&quot;,\n  \\&quot;email\\&quot;: \\&quot;alice@example.com\\&quot;,\n  \\&quot;datetime\\&quot;: \\&quot;2025-01-01T12:00:00Z\\&quot;,\n  \\&quot;guests\\&quot;: [\n    { \\&quot;name\\&quot;: \\&quot;Alice\\&quot;, \\&quot;age\\&quot;: 25 },\n    { \\&quot;name\\&quot;: \\&quot;Bob\\&quot;, \\&quot;age\\&quot;: 30, \\&quot;special_needs\\&quot;: \\&quot;Vegan\\&quot; }\n  ],\n  \\&quot;table\\&quot;: { \\&quot;type\\&quot;: \\&quot;specific\\&quot;, \\&quot;table_number\\&quot;: 1 },\n  \\&quot;notes\\&quot;: null\n}\n&quot;\n\npub fn main() {\n  let assert Ok(data) = json.decode(data, dynamic.dynamic)\n\n  data |&gt; toy.decode(reservation_decoder()) |&gt; io.debug\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/toy&gt;.\n\n","url":"index.html"},{"doc":"toy","title":"toy","content":"# Quickstart\n\nWelcome to the Toy quickstart guide!\n\nThis guide will walk you through the basics of using Toy to decode and validate\ndynamic data in Gleam.\n\n## Installation\n\nToy is available on [Hex](https://hex.pm/packages/toy) and can be installed\nwith the following command:\n\n```sh\ngleam add toy\n```\n\n&gt; Note: If you are using a library that needs a `dynamic.Decoder(a)` and you\n&gt; want to use Toy, you can use the `toy.to_stdlib_decoder` function to convert\n&gt; the decoder to a `dynamic.Decoder(a)`.\n\n## Rule number one of Toy\n\n**Don not perform side effects in the decoder**. I repeat, **do not perform side\neffects in the decoder**.\n\n```gleam\nimport toy\nimport gleam/dynamic\nimport gleam/dict\n\npub type User {\n  User(name: String, age: Int)\n}\n\npub fn user_decoder() {\n  use name &lt;- toy.field(&quot;name&quot;, toy.string)\n  use age &lt;- toy.field(&quot;age&quot;, toy.int)\n\n  // DO NOT DO THIS\n  log.info(name)\n  // DO NOT DO THIS\n  db.save_user(name, age)\n\n  toy.decoded(User(name:, age:))\n}\n\npub fn main() {\n  let assert Ok(user) = dict.from_list([\n    #(&quot;name&quot;, dynamic.from(&quot;Alice&quot;)),\n    #(&quot;age&quot;, dynamic.from(42)),\n  ])\n  |&gt; dynamic.from\n  |&gt; toy.decode(user_decoder())\n\n  // Do this\n  log.info(user.name)\n  // Do this\n  db.save_user(user.name, user.age)\n}\n```\n\nThe reason for this rule is simple. Every line of your decoder will be executed\nalways. If the name isn&#39;t a string, the decoder will continue to the next line\nand check the age, and so on. This allows us to return errors for all fields at\nonce.\n\nIf name is not a string, the name variable will be set to a &quot;default&quot; value. In\ncase of a string, this is a hardcoded empty string `&quot;&quot;`. In case of an int, it\nis 0. etc.\n\nSo if you perform any side effects in the decoder, you will be presented with\nunvalidated data. If you need to do something with the data, you should decode it\nfirst. See the example above.\n\n## Decoding a record\n\nOkay, so, now that we have that out of the way, let&#39;s get to the meat of Toy.\nIn real applications, it is very common to decode a record. Be it from a JSON\npayload, a database row, or a configuration file. Toy allows you to do this very\neasily. Let&#39;s take a look at a simple user decoder.\n\n```gleam\npub fn user_decoder() {\n  use name &lt;- toy.field(&quot;name&quot;, toy.string)\n  use age &lt;- toy.field(&quot;age&quot;, toy.int)\n\n  toy.decoded(User(name:, age:))\n}\n\npub fn main() {\n  dict.from_list([\n    #(&quot;name&quot;, dynamic.from(&quot;Alice&quot;)),\n    #(&quot;age&quot;, dynamic.from(42)),\n  ])\n  |&gt; dynamic.from\n  |&gt; toy.decode(user_decoder())\n  |&gt; should.equal(Ok(User(name: &quot;Alice&quot;, age: 42)))\n}\n```\n\nOur user has a *name* and an *age*. We use the `use` keyword along with the `field`\nfunction that toy provides for us. The `field` function takes two arguments.\n\nThe first argument can by &quot;anything&quot;. Toy will then attempt to index into the\ndynamic value with the provided key. Most of the time, you will pass in a string\nor an int as the key. In this case, we are trying to decode an object. Therefore\nwe pass the key `&quot;name&quot;` as a string.\n\nThe second argument is the decoder to use. Toy provides a number of decoders out\nof the box. But they are not special at all. You could pass in your own decoder\nas well. In fact, our function `user_decoder` creates a decoder, same as in Toy!\nSo we can easily compose decoders together.\n\n```gleam\npub fn account_decoder() {\n    use type_ &lt;- toy.field(&quot;type&quot;, toy.string)\n    use user &lt;- toy.field(&quot;user&quot;, user_decoder())\n\n    toy.decoded(Account(type_:, user:))\n}\n```\n\nSee! We can decode a record with a nested record just as easily as we can decode\na our user record. This is one of the goals of Toy. It aims to be *Composable*.\n\nLast thing in our record decoding journey is the `toy.decoded` function. This\nfunction isn&#39;t anything special. It creates a decoder that will always succeed\nand return the provided value. The reason for it are the `use` expressions.\nHowever, you don&#39;t have to worry about it for now, just remember to wrap your\nfinal decoded value in this function.\n\n&gt; Also, notice how we use the label shorthand syntax to set the fields on\nthe final record. When you use this feature, you don&#39;t have to worry about the\norder of arguments passed into the final record. Gleam will automatically assign\nthe `name` value to the `name` field in the record. If you use this shorthand\nin all of your decoders (and we highly recommend it), you minimize the chance of\nmessing up the order of fields in the record, let&#39;s say during refactoring,\nor developing a new feature.\n\n## Different decoding based on a value\n\nIn real applications data is often complicated and varied. Let&#39;s say the user\nsends us their pet preference as a string. How can we map this string into a\ntype? Or what if their pets have different attributes each? We need to decode\nthe correct attributes based on the pet type.\n\n### Simple enum\n\nToy provides two main ways of decoding an Enum. If your enum is simple, you can\nuse the `toy.enum` function. The function is very simple. It takes a list of\nmappings from the source value to the target value.\n\n```gleam\npub type Pet {\n  Dog\n  Cat\n  Fish\n}\n\npub fn main() {\n  let decoder = toy.string |&gt; toy.enum([\n    #(&quot;dog&quot;, Dog),\n    #(&quot;cat&quot;, Cat),\n    #(&quot;fish&quot;, Fish),\n  ])\n\n  let data = dynamic.from(&quot;dog&quot;)\n\n  toy.decode(data, decoder)\n  |&gt; should.equal(Ok(Dog))\n}\n```\n\nYou may notice it is generic. Therefore you can use it over any value. Not just\nstrings.\n\n```gleam\npub fn main() {\n  let decoder = toy.int |&gt; toy.enum([\n    #(1, Dog),\n    #(2, Cat),\n    #(3, Fish),\n  ])\n\n  let data = dynamic.from(2)\n\n  toy.decode(data, decoder)\n  |&gt; should.equal(Ok(Cat))\n}\n```\n\n### Advanced enum\n\nIf your enum is more complicated, you may want to implement a custom decoder.\nDon&#39;t worry, it is very simple.\n\n```gleam\npub type Pet {\n  Dog(tag: String)\n  Cat(collar: String)\n  Fish(color: String)\n}\n\npub fn pet_decoder() {\n  use type_ &lt;- toy.field(&quot;type&quot;, toy.string)\n\n  case type_ {\n    &quot;dog&quot; -&gt; {\n      use tag &lt;- toy.field(&quot;tag&quot;, toy.string)\n      toy.decoded(Dog(tag:))\n    }\n    &quot;cat&quot; -&gt; {\n      use collar &lt;- toy.field(&quot;collar&quot;, toy.string)\n      toy.decoded(Cat(collar:))\n    }\n    &quot;fish&quot; -&gt; {\n      use color &lt;- toy.field(&quot;color&quot;, toy.string)\n      toy.decoded(Fish(color:))\n    }\n    _ -&gt; toy.fail(toy.InvalidType(&quot;Pet&quot;, type_), Dog(&quot;&quot;))\n  }\n}\n```\n\nYou may notice that we pass in a `Dog(&quot;&quot;)` value to the `fail` function.\nThis dog will not escape this decoder. It is a default value that will be\nreturned if this decoder fails, so we can continue to decode the rest of the\ndata.\n\nYou may also ask, *How do I know if the &quot;type_&quot; field is a real value or a default?*.\nThe simple answer is that you don&#39;t. And you don&#39;t need to. You can always treat\nthe value as if it were valid. If it isn&#39;t valid, the field function that decoded\nit will return an error. So you can safely use the values and rely on the upstream\ndecoders to return errors if they fail.\n\n### Enum without discriminant\n\nSo, your API returns a value which can have many shapes, but no field can be\nused to differentiate between them. Don&#39;t worry, Toy provides a solution for you.\nThe `toy.one_of` function takes a list of decoders and attempts to decode the\nvalue with each of the decoders in order. The first successful one will be\nreturned. If none of the decoders are successful, all errors will be returned.\nIf you want to consolidate the errors, or just change them to something else,\nyou can use the `toy.map_errors` function.\n\n```gleam\npub type Pet {\n  Dog(tag: String)\n  Cat(collar: String)\n  Fish(color: String)\n}\n\npub fn dog_decoder() {\n  use tag &lt;- toy.field(&quot;tag&quot;, toy.string)\n  toy.decoded(Dog(tag:))\n}\n\npub fn cat_decoder() {\n  use collar &lt;- toy.field(&quot;collar&quot;, toy.string)\n  toy.decoded(Cat(collar:))\n}\n\npub fn fish_decoder() {\n  use color &lt;- toy.field(&quot;color&quot;, toy.string)\n  toy.decoded(Fish(color:))\n}\n\npub fn main() {\n  let decoder = toy.one_of([dog_decoder(), cat_decoder(), fish_decoder()])\n\n  let data = dynamic.from(dict.from_list([\n      #(&quot;tag&quot;, dynamic.from(&quot;woof&quot;)),\n  ]))\n\n  toy.decode(data, decoder)\n  |&gt; should.equal(Ok(Dog(tag: &quot;woof&quot;)))\n}\n```\n\n## Validation\n\nToy is not just a decoding library. It also provides validation functions.\nThis makes it easy to write one decoder, and be confidend that you data has \nthe right shape, and values are within the expected limits.\n\nValidation in Toy is done &quot;one by one&quot;. This means that if one validation fails,\nthe others won&#39;t be executed.\n\nLet&#39;s validate the age field of our user decoder.\n\n```gleam\npub fn user_decoder() {\n  use name &lt;- toy.field(&quot;name&quot;, toy.string)\n  use age &lt;- toy.field(&quot;age&quot;, toy.int |&gt; toy.int_min(18))\n\n  toy.decoded(User(name:, age:))\n}\n```\n\nIt&#39;s simple! You just pipe the decoder to the validation function and that&#39;s it.\n\n### Custom validation (refine)\n\ntoy doesn&#39;t contain all the validation functions that you might need. So we\nprovide you with the facility to create your own validation functions.\nOne of these is the `toy.refine` function, which has been inspired by the\n[Zod](https://github.com/colinhacks/zod) typescript library.\n\n```gleam\npub fn user_decoder() {\n  use name &lt;- toy.field(&quot;name&quot;, toy.string |&gt; toy.refine(fn(name) {\n    case name {\n      &quot;toy&quot; -&gt; Error([toy.ToyError(toy.ValidationFailed(&quot;name_taken&quot;, &quot;new_name&quot;, name), [])])\n      _ -&gt; Ok(Nil)\n    }\n  }))\n toy.decoded(User(name:))\n}\n```\n\n\n## Modifying the result\n\nAs you go by your day as a happy Gleam developer, you will find yourself in a\nsituation, where those pesky (Insert other language) devs return an object where\nthe value might be `null` or `undefined`. Toy provides you with the `toy.nullable`\nfunction, which you can use exactly like any other validator. But this function\nwill actually modify the result of the decoder.\n\n```gleam\npub fn user_decoder() {\n  use name &lt;- toy.field(&quot;name&quot;, toy.string |&gt; toy.string_min(1) |&gt; toy.nullable)\n  toy.decoded(User(name:))\n}\n```\n\nNotice, that the `toy.string_min(1)` function is applied before the `toy.nullable`.\nThis must be done this way. If you were to put it after the `toy.nullable`, the\ncompiler would complain, because after the `toy.nullable` function, the result\nof the decoder is `Option(String)`.\n\nThis is not the only function that allows for this behavior. You can\nuse the `toy.map` or `toy.try_map` functions to create your own custom modifiers.\n\n&gt; There is one important difference between `toy.nullable` and `toy.map`. They\nhave different execution orders. The `toy.nullable` takes the dynamic value,\nchecks if it is `null` and if not, it passed it onto the next decoder.\n`toy.map` on the other hand, takes the dynamic value, passes it onto the next\ndecoder, and then applies the function to the result of the decoder.\nThis enables the nice piping property of Toy&#39;s validation functions.\nYou will hopefully never run into this, but if you do, now you know.\n\n\n","url":"quickstart.html"},{"doc":"toy","title":"Decoder","content":"pub type Decoder(a) {\n  Decoder(\n    run: fn(dynamic.Dynamic) -&gt; #(a, Result(a, List(ToyError))),\n  )\n}\n A decoder is a function that takes a `Dynamic` value and returns a tuple\n containing the default value of the same type and a `Result` with the decoded\n value or a list of errors\nDecoder(\n  run: fn(dynamic.Dynamic) -&gt; #(a, Result(a, List(ToyError))),\n)\n\n\nSynonyms:\ntoy.Decoder\ntoy Decoder","url":"toy.html#Decoder"},{"doc":"toy","title":"ToyError","content":"pub type ToyError {\n  ToyError(error: ToyFieldError, path: List(String))\n}\n Contains decoding or validation errors\nToyError(error: ToyFieldError, path: List(String))\n\n\nSynonyms:\ntoy.ToyError\ntoy ToyError","url":"toy.html#ToyError"},{"doc":"toy","title":"ToyFieldError","content":"pub type ToyFieldError {\n  InvalidType(expected: String, found: String)\n  Missing(expected: String)\n  ValidationFailed(\n    check: String,\n    expected: String,\n    found: String,\n  )\n  AllFailed(failures: List(List(ToyError)))\n}\n Each type of error that can be returned by the decoders\nInvalidType(expected: String, found: String)\n\n\nMissing(expected: String)\n\n\nValidationFailed(check: String, expected: String, found: String)\n\n\nAllFailed(failures: List(List(ToyError)))\n\n\nSynonyms:\ntoy.ToyFieldError\ntoy ToyFieldError","url":"toy.html#ToyFieldError"},{"doc":"toy","title":"base16_string","content":"pub const base16_string: Decoder(BitArray)\n Decodes a `BitArray` from a base16 encoded string\nSynonyms:\ntoy.base16_string\ntoy base16_string","url":"toy.html#base16_string"},{"doc":"toy","title":"base64_string","content":"pub const base64_string: Decoder(BitArray)\n Decodes a `BitArray` from a base64 encoded string\nSynonyms:\ntoy.base64_string\ntoy base64_string","url":"toy.html#base64_string"},{"doc":"toy","title":"base64_url_string","content":"pub const base64_url_string: Decoder(BitArray)\n Decodes a `BitArray` from a url safe base64 encoded string\n using `-` instead of `+` and `_` instead of `/`\nSynonyms:\ntoy.base64_url_string\ntoy base64_url_string","url":"toy.html#base64_url_string"},{"doc":"toy","title":"bit_array","content":"pub const bit_array: Decoder(BitArray)\n Decode a `BitArray`\nSynonyms:\ntoy.bit_array\ntoy bit_array","url":"toy.html#bit_array"},{"doc":"toy","title":"bool","content":"pub const bool: Decoder(Bool)\n Decode a `Bool` value\nSynonyms:\ntoy.bool\ntoy bool","url":"toy.html#bool"},{"doc":"toy","title":"bool_string","content":"pub const bool_string: Decoder(Bool)\n Decodes a `String` and parses it as `Bool` with the given variants:\n \n case value {\n   &quot;True&quot; | &quot;true&quot; -&gt; Ok(True)\n   &quot;False&quot; | &quot;false&quot; -&gt; Ok(False)\n   _ -&gt; Error(Nil)\n }\n \n In case pattern matching on the string fails,\n returns `ValidationFailed` error\n **Error type**: `bool_string`\nSynonyms:\ntoy.bool_string\ntoy bool_string","url":"toy.html#bool_string"},{"doc":"toy","title":"dynamic","content":"pub const dynamic: Decoder(Dynamic)\n Always decodes the provided value as `Dynamic`.\n Error is never returned from this decoder\nSynonyms:\ntoy.dynamic\ntoy dynamic","url":"toy.html#dynamic"},{"doc":"toy","title":"float","content":"pub const float: Decoder(Float)\n Decode a `Float` value\nSynonyms:\ntoy.float\ntoy float","url":"toy.html#float"},{"doc":"toy","title":"float_string","content":"pub const float_string: Decoder(Float)\n Decode a `String` and parse it as `Float`\n Allows for values that look like floats &quot;1.345&quot; and integers &quot;12345&quot;\n In case parsing failed, returns `ValidationFailed` error\n\n **Error type**: `float_string`\nSynonyms:\ntoy.float_string\ntoy float_string","url":"toy.html#float_string"},{"doc":"toy","title":"int","content":"pub const int: Decoder(Int)\n Decode an `Int` value\nSynonyms:\ntoy.int\ntoy int","url":"toy.html#int"},{"doc":"toy","title":"int_string","content":"pub const int_string: Decoder(Int)\n Decode a `String` and parse it as `Int`\n In case parsing failed, returns `ValidationFailed` error\n **Error type**: `int_string`\nSynonyms:\ntoy.int_string\ntoy int_string","url":"toy.html#int_string"},{"doc":"toy","title":"nullish","content":"pub const nullish: Decoder(Nil)\n Decodes a *nullish* value.\n In erlang it is one of these atoms: `undefined`, `null`, `nil`.\n In javascript it is one of these values: `undefined`, `null`\nSynonyms:\ntoy.nullish\ntoy nullish","url":"toy.html#nullish"},{"doc":"toy","title":"string","content":"pub const string: Decoder(String)\n Decode a `String` value\nSynonyms:\ntoy.string\ntoy string","url":"toy.html#string"},{"doc":"toy","title":"uri","content":"pub const uri: Decoder(Uri)\n Decodes a `Uri` from a string\n Uses the `gleam/uri` module on erlang and the `URL` object on javascript\n\n **Error type**: `uri`\nSynonyms:\ntoy.uri\ntoy uri","url":"toy.html#uri"},{"doc":"toy","title":"decode","content":"pub fn decode(\n  data: Dynamic,\n  decoder: Decoder(a),\n) -&gt; Result(a, List(ToyError))\n Takes a `Dynamic` value and runs a `Decoder` on it, returning the result\n of the decoding process\nSynonyms:\ntoy.decode\ntoy decode","url":"toy.html#decode"},{"doc":"toy","title":"decoded","content":"pub fn decoded(value: a) -&gt; Decoder(a)\n Creates a decoder which directly returns the provided value.\n It is useful when decoding records.\n\n \n pub fn user_decoder() {\n   use name &lt;- toy.field(&quot;name&#39;, toy.string)\n   toy.decoded(User(name:))\n }\n \nSynonyms:\ntoy.decoded\ntoy decoded","url":"toy.html#decoded"},{"doc":"toy","title":"dict","content":"pub fn dict(\n  key_type: Decoder(a),\n  value_type: Decoder(b),\n) -&gt; Decoder(Dict(a, b))\n Decodes a `Dict` using the provided key and value decoders\nSynonyms:\ntoy.dict\ntoy dict","url":"toy.html#dict"},{"doc":"toy","title":"enum","content":"pub fn enum(\n  dec: Decoder(a),\n  variants: List(#(a, b)),\n) -&gt; Decoder(b)\n Takes any decoder and a list of mappings from the decoded value to a new\n value. Returns the new value corresponding to the decoded value.\n\n In case of an error, return a `ValidationFailed` with the expected type\n being a list of the possible values separated by a comma.\n\n **Panics if the list of variants is empty**\n\n **Error type**: `enum`\n\n \n pub type Fish {\n   Salmon\n   Trout\n   Shard\n   Cod\n }\n\n let decoder =\n   toy.string\n   |&gt; toy.enum([\n     #(&quot;salmon&quot;, Salmon),\n     #(&quot;trout&quot;, Trout),\n     #(&quot;shard&quot;, Shard),\n     #(&quot;cod&quot;, Cod),\n   ])\n\n dynamic.from(&quot;salmon&quot;)\n |&gt; toy.decode(decoder)\n |&gt; should.equal(Ok(Salmon))\n \nSynonyms:\ntoy.enum\ntoy enum","url":"toy.html#enum"},{"doc":"toy","title":"fail","content":"pub fn fail(error: ToyFieldError, default: a) -&gt; Decoder(a)\n Returns a decoder that always fails with the provided error\nSynonyms:\ntoy.fail\ntoy fail","url":"toy.html#fail"},{"doc":"toy","title":"field","content":"pub fn field(\n  key: a,\n  decoder: Decoder(b),\n  next: fn(b) -&gt; Decoder(c),\n) -&gt; Decoder(c)\n Decode a field from a `Dynamic` value\n\n This function will index into dictionary with any key type, tuples with\n integer or javascript arrays and objects. The value found under the key\n will be decoded with the provided decoder.\n\n \n pub fn user_decoder() {\n   use name &lt;- toy.field(&quot;name&quot;, toy.string)\n   toy.decoded(User(name:))\n }\n \nSynonyms:\ntoy.field\ntoy field","url":"toy.html#field"},{"doc":"toy","title":"float_max","content":"pub fn float_max(\n  dec: Decoder(Float),\n  maximum: Float,\n) -&gt; Decoder(Float)\n Validates that number is less than the provided maximum\n\n **Error type**: `float_max`\nSynonyms:\ntoy.float_max\ntoy float_max","url":"toy.html#float_max"},{"doc":"toy","title":"float_min","content":"pub fn float_min(\n  dec: Decoder(Float),\n  minimum: Float,\n) -&gt; Decoder(Float)\n Validates that number is greater or equal to the provided minimum\n\n **Error type**: `float_min`\nSynonyms:\ntoy.float_min\ntoy float_min","url":"toy.html#float_min"},{"doc":"toy","title":"float_range","content":"pub fn float_range(\n  dec: Decoder(Float),\n  minimum: Float,\n  maximum: Float,\n) -&gt; Decoder(Float)\n Validates that the number is withing the provided range [minimum, maximum)\n\n **Error type**: `float_range`\nSynonyms:\ntoy.float_range\ntoy float_range","url":"toy.html#float_range"},{"doc":"toy","title":"int_max","content":"pub fn int_max(dec: Decoder(Int), maximum: Int) -&gt; Decoder(Int)\n Validates that number is less than the provided maximum\n\n **Error type**: `int_max`\nSynonyms:\ntoy.int_max\ntoy int_max","url":"toy.html#int_max"},{"doc":"toy","title":"int_min","content":"pub fn int_min(dec: Decoder(Int), minimum: Int) -&gt; Decoder(Int)\n Validates that number is greater or equal to the provided minimum\n\n **Error type**: `int_min`\nSynonyms:\ntoy.int_min\ntoy int_min","url":"toy.html#int_min"},{"doc":"toy","title":"int_range","content":"pub fn int_range(\n  dec: Decoder(Int),\n  minimum: Int,\n  maximum: Int,\n) -&gt; Decoder(Int)\n Validates that number is in the provided range: [minimum, maximum)\n\n **Error type**: `int_range`\nSynonyms:\ntoy.int_range\ntoy int_range","url":"toy.html#int_range"},{"doc":"toy","title":"is_equal","content":"pub fn is_equal(dec: Decoder(a), literal: a) -&gt; Decoder(a)\n Validates that the decoded value is equal to the provided value\n The comparison is made using the `==` operator\n **Error type**: `is_literal`\nSynonyms:\ntoy.is_equal\ntoy is_equal","url":"toy.html#is_equal"},{"doc":"toy","title":"list","content":"pub fn list(item: Decoder(a)) -&gt; Decoder(List(a))\n Decode a list of values\n\n \n pub fn fruits_decoder() {\n   toy.list({\n     use name &lt;- toy.field(&quot;name&quot;, toy.string)\n     toy.decoded(Fruit(name:))\n   })\n }\nSynonyms:\ntoy.list\ntoy list","url":"toy.html#list"},{"doc":"toy","title":"list_max","content":"pub fn list_max(\n  dec: Decoder(List(a)),\n  maximum: Int,\n) -&gt; Decoder(List(a))\n Validates that the length of the list is less than the provided maximum\n\n **Error type**: `list_max`\nSynonyms:\ntoy.list_max\ntoy list_max","url":"toy.html#list_max"},{"doc":"toy","title":"list_min","content":"pub fn list_min(\n  dec: Decoder(List(a)),\n  minimum: Int,\n) -&gt; Decoder(List(a))\n Validates that the length of the list is at least the provided number\n\n **Error type**: `list_min`\nSynonyms:\ntoy.list_min\ntoy list_min","url":"toy.html#list_min"},{"doc":"toy","title":"list_nonempty","content":"pub fn list_nonempty(dec: Decoder(List(a))) -&gt; Decoder(List(a))\n Validates that the list is not empty (contains at least one element)\n\n **Error type**: `list_nonempty`\nSynonyms:\ntoy.list_nonempty\ntoy list_nonempty","url":"toy.html#list_nonempty"},{"doc":"toy","title":"map","content":"pub fn map(dec: Decoder(a), fun: fn(a) -&gt; b) -&gt; Decoder(b)\n Map the result of the decoder to a new value\n\n \n pub type Unit {\n   Centimeters(Int)\n   Milimeters(Int)\n }\n\n pub type User {\n   User(height: Unit)\n }\n\n pub fn user_decoder() {\n   use height &lt;- toy.field(&quot;height&quot;, toy.int |&gt; toy.map(Centimeters))\n   toy.decoded(User(:height))\n }\nSynonyms:\ntoy.map\ntoy map","url":"toy.html#map"},{"doc":"toy","title":"map_errors","content":"pub fn map_errors(\n  dec: Decoder(a),\n  fun: fn(List(ToyError)) -&gt; List(ToyError),\n) -&gt; Decoder(a)\n If the passed in decoder returns an error, the provided function is called\n to allow you to change or swap the errors\nSynonyms:\ntoy.map_errors\ntoy map_errors","url":"toy.html#map_errors"},{"doc":"toy","title":"nullable","content":"pub fn nullable(of dec: Decoder(a)) -&gt; Decoder(Option(a))\n Creates a new decoder from an existing one, which will return `None` if\n the value is `null` or `undefined` on javascript, or `nil`, `null`,\n `undefined` on erlang. Otherwise it will return the result of the provided\n decoder wrapped in `Some`\nSynonyms:\ntoy.nullable\ntoy nullable","url":"toy.html#nullable"},{"doc":"toy","title":"one_of","content":"pub fn one_of(decoders: List(Decoder(a))) -&gt; Decoder(a)\n Attempts to decode the value with each of the decoders in order. The first\n successful one will be returned. If none of the decoders are successful,\n an error is returned specifying possible options.\n\n This function will panic if the list of decoders is empty.\n\n \n let dog_decoder = fn() {\n   use tag &lt;- toy.field(&quot;tag&quot;, toy.string)\n   toy.decoded(Dog(tag:))\n }\n\n let cat_decoder = fn() {\n   use collar &lt;- toy.field(&quot;collar&quot;, toy.string)\n   toy.decoded(Cat(collar:))\n }\n\n let fish_decoder = fn() {\n   use color &lt;- toy.field(&quot;color&quot;, toy.string)\n   toy.decoded(Fish(color:))\n }\n\n let decoder = toy.one_of([dog_decoder(), cat_decoder(), fish_decoder()])\n\n dict.from_list([#(&quot;tag&quot;, dynamic.from(&quot;woof&quot;))])\n |&gt; dynamic.from\n |&gt; toy.decode(decoder)\n |&gt; should.equal(Ok(Dog(tag: &quot;woof&quot;)))\n\n dict.from_list([#(&quot;feathers&quot;, dynamic.from(&quot;blue&quot;))])\n |&gt; dynamic.from\n |&gt; toy.decode(decoder)\n |&gt; should.equal(\n   Error([\n     toy.ToyError(\n       toy.AllFailed([\n         [toy.ToyError(toy.Missing(&quot;String&quot;), [&quot;\\&quot;tag\\&quot;&quot;])],\n         [toy.ToyError(toy.Missing(&quot;String&quot;), [&quot;\\&quot;collar\\&quot;&quot;])],\n         [toy.ToyError(toy.Missing(&quot;String&quot;), [&quot;\\&quot;color\\&quot;&quot;])],\n       ]),\n       [],\n     ),\n   ]),\n )\n \nSynonyms:\ntoy.one_of\ntoy one_of","url":"toy.html#one_of"},{"doc":"toy","title":"option","content":"pub fn option(of dec: Decoder(a)) -&gt; Decoder(Option(a))\n Decodes a gleam `Option` type. In erlang represented as `{ok, Value}` or `none`.\n In javascript represented as an instance of `Some` or `None` classes.\nSynonyms:\ntoy.option\ntoy option","url":"toy.html#option"},{"doc":"toy","title":"optional_field","content":"pub fn optional_field(\n  key: a,\n  decoder: Decoder(b),\n  next: fn(Option(b)) -&gt; Decoder(c),\n) -&gt; Decoder(c)\n Decode a field from a `Dynamic` value\n\n This function will index into dictionary with any key type, tuples with\n integer or javascript arrays and objects. The value found under the key\n will be decoded with the provided decoder.\n\n `None` is returned only if the field is missing. Otherwise the provided\n decoder is used to decode the value.\n\n \n pub fn reservation_decoder() {\n   use note &lt;- toy.optional_field(&quot;note&quot;, toy.string)\n   toy.decoded(User(name:))\n }\n \nSynonyms:\ntoy.optional_field\ntoy optional_field","url":"toy.html#optional_field"},{"doc":"toy","title":"optional_subfield","content":"pub fn optional_subfield(\n  keys: List(a),\n  decoder: Decoder(b),\n  next: fn(Option(b)) -&gt; Decoder(c),\n) -&gt; Decoder(c)\n Same as `optional_field` but indexes recursively with the provided keys\n \n pub fn user_decoder() {\n   use name &lt;- toy.optional_subfield([&quot;person&quot;, &quot;name&quot;], toy.string)\n   toy.decoded(User(name:))\n }\n \nSynonyms:\ntoy.optional_subfield\ntoy optional_subfield","url":"toy.html#optional_subfield"},{"doc":"toy","title":"refine","content":"pub fn refine(\n  dec: Decoder(a),\n  fun: fn(a) -&gt; Result(Nil, List(ToyError)),\n) -&gt; Decoder(a)\n Refine the result of the decoder with a validation function\n\n \n pub fn user_decoder() {\n   use name &lt;- toy.field(&quot;name&quot;, toy.string |&gt; toy.refine(fn(name) {\n     case name {\n       &quot;toy&quot; -&gt; Error([toy.ToyError(toy.ValidationFailed(&quot;name_taken&quot;, &quot;new_name&quot;, name), [])])\n       _ -&gt; Ok(Nil)\n     }\n   }))\n  toy.decoded(User(name:))\n }\nSynonyms:\ntoy.refine\ntoy refine","url":"toy.html#refine"},{"doc":"toy","title":"string_email","content":"pub fn string_email(dec: Decoder(String)) -&gt; Decoder(String)\n Validates that the string contains an email address.\n This is done by checking if the string contains the &quot;@&quot; character.\n\n **Error type**: `string_email`\nSynonyms:\ntoy.string_email\ntoy string_email","url":"toy.html#string_email"},{"doc":"toy","title":"string_max","content":"pub fn string_max(\n  dec: Decoder(String),\n  maximum: Int,\n) -&gt; Decoder(String)\n Validates that the length of the string is less than the provided number\n\n **Error type**: `string_max`\nSynonyms:\ntoy.string_max\ntoy string_max","url":"toy.html#string_max"},{"doc":"toy","title":"string_min","content":"pub fn string_min(\n  dec: Decoder(String),\n  minimum: Int,\n) -&gt; Decoder(String)\n Validates that the length of the string is at least the provided number\n\n **Error type**: `string_min`\nSynonyms:\ntoy.string_min\ntoy string_min","url":"toy.html#string_min"},{"doc":"toy","title":"string_nonempty","content":"pub fn string_nonempty(dec: Decoder(String)) -&gt; Decoder(String)\n Validates that the string contains some characters that are not whitespace.\n\n **Error type**: `string_nonempty`\nSynonyms:\ntoy.string_nonempty\ntoy string_nonempty","url":"toy.html#string_nonempty"},{"doc":"toy","title":"string_uri","content":"pub fn string_uri(dec: Decoder(String)) -&gt; Decoder(String)\n Validates that the string is a valid uri\n Uses the `gleam/uri` module on erlang and the `URL` object on javascript\n\n **Error type**: `string_uri`\nSynonyms:\ntoy.string_uri\ntoy string_uri","url":"toy.html#string_uri"},{"doc":"toy","title":"subfield","content":"pub fn subfield(\n  keys: List(a),\n  decoder: Decoder(b),\n  next: fn(b) -&gt; Decoder(c),\n) -&gt; Decoder(c)\n Same as `field` but indexes recursively with the provided keys\n \n pub fn user_decoder() {\n   use name &lt;- toy.subfield([&quot;person&quot;, &quot;name&quot;], toy.string)\n   toy.decoded(User(name:))\n }\n \nSynonyms:\ntoy.subfield\ntoy subfield","url":"toy.html#subfield"},{"doc":"toy","title":"to_stdlib_decoder","content":"pub fn to_stdlib_decoder(\n  dec: Decoder(a),\n) -&gt; fn(Dynamic) -&gt; Result(a, List(DecodeError))\n Converts a `toy.Decoder(a)` to a `dynamic.Decoder(a)`\n You shouldn&#39;t use this function if you can avoid it.\n But sometimes libraries expect a `dynamic.Decoder(a)` and passing\n `dynamic.dynamic` just makes things hard.\n\n \n import gleam/dynamic\n\n pub fn user_decoder() {\n   use name &lt;- toy.field(&quot;name&quot;, toy.string)\n   toy.decoded(User(name:))\n }\n\n pub fn main() {\n let assert Ok(response) = \n   pgo.execute(sql, db, [pgo.int(1)], toy.to_stdlib_decoder(user_decoder()))\n }\n \n\n *Mapping of `toy.ToyError` into `dynamic.DecodeError`*\n - `toy.InvalidType(&quot;Int&quot;, &quot;String&quot;)` -&gt; `dynamic.DecodeError(&quot;Int&quot;, &quot;String&quot;, path)`\n - `toy.Missing(&quot;Int&quot;)` -&gt; `dynamic.DecodeError(&quot;Int&quot;, &quot;Nothing&quot;, path)`\n - `toy.ValidationFailed(&quot;int_min&quot;, &quot;&gt;=10&quot;, &quot;7&quot;)` -&gt; `dynamic.DecodeError(&quot;int_min__&gt;=10, int_min__7, path)`\n - `toy.AllFailed([...])` -&gt; `dynamic.DecodeError(&quot;AllFailed&quot;, &quot;AllFailed&quot;, path)`\n    and all the errors in the list translated recursively\nSynonyms:\ntoy.to_stdlib_decoder\ntoy to_stdlib_decoder","url":"toy.html#to_stdlib_decoder"},{"doc":"toy","title":"try_map","content":"pub fn try_map(\n  dec: Decoder(a),\n  default: b,\n  fun: fn(a) -&gt; Result(b, List(ToyError)),\n) -&gt; Decoder(b)\n Map the result of the decoder to a new value or return an error\n\n \n pub fn user_decoder() {\n   use name &lt;- toy.field(&quot;name&quot;, toy.string |&gt; toy.try_map(&quot;&quot;, fn(name) {\n     case name {\n       &quot;toy&quot; -&gt; Error([toy.ToyError(toy.ValidationFailed(&quot;name_taken&quot;, &quot;new_name&quot;, name), [])])\n       _ -&gt; Ok(string.uppercase(name))\n     }\n   }))\n  toy.decoded(User(name:))\n }\nSynonyms:\ntoy.try_map\ntoy try_map","url":"toy.html#try_map"},{"doc":"toy","title":"toy","content":"","url":"toy.html"}]);